"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var THREE = _interopRequireWildcard(require("three"));

var _TiledGeometryLayer2 = _interopRequireDefault(require("../../../Layer/TiledGeometryLayer"));

var _Extent = _interopRequireDefault(require("../../Geographic/Extent"));

var _PanoramaTileBuilder = _interopRequireDefault(require("./PanoramaTileBuilder"));

var _Constants = _interopRequireDefault(require("./Constants"));

var textureSize = new THREE.Vector2(512, 256);
var center = new THREE.Vector3();

var PanoramaLayer =
/*#__PURE__*/
function (_TiledGeometryLayer) {
  (0, _inherits2.default)(PanoramaLayer, _TiledGeometryLayer);

  /**
   * A {@link TiledGeometryLayer} to use with a {@link PanoramaView}. It has
   * specific method for updating and subdivising its grid.
   *
   * @constructor
   * @extends TiledGeometryLayer
   *
   * @param {string} id - The id of the layer, that should be unique. It is
   * not mandatory, but an error will be emitted if this layer is added a
   * {@link View} that already has a layer going by that id.
   * @param {Coordinates} coordinates - The coordinates of the origin of the
   * panorama.
   * @param {number} type - The type of projection for the panorama: 1 for a
   * cylindrical projection, 2 for a spherical one.
   * @param {Object} [config] - Optional configuration, all elements in it
   * will be merged as is in the layer. For example, if the configuration
   * contains three elements <code>name, protocol, extent</code>, these
   * elements will be available using <code>layer.name</code> or something
   * else depending on the property name.
   * @param {THREE.Object3d} [config.object3d=THREE.Group] - The object3d used to
   * contain the geometry of the TiledGeometryLayer. It is usually a
   * <code>THREE.Group</code>, but it can be anything inheriting from a
   * <code>THREE.Object3d</code>.
   * @param {number} [config.maxSubdivisionLevel=10] - Maximum subdivision
   * level for this tiled layer.
   * @param {number} [config.ratio=1] - Ratio for building the panorama
   * sphere.
   *
   * @throws {Error} <code>object3d</code> must be a valid
   * <code>THREE.Object3d</code>.
   */
  function PanoramaLayer(id, coordinates, type, config) {
    var _this;

    (0, _classCallCheck2.default)(this, PanoramaLayer);
    var schemeTile;

    if (type === _Constants.default.SPHERICAL) {
      // equirectangular -> spherical geometry
      schemeTile = [new _Extent.default('EPSG:4326', {
        west: -180,
        east: 0,
        north: 90,
        south: -90
      }), new _Extent.default('EPSG:4326', {
        west: 0,
        east: 180,
        north: 90,
        south: -90
      })];
    } else if (type === _Constants.default.CYLINDRICAL) {
      // cylindrical geometry
      schemeTile = [new _Extent.default('EPSG:4326', {
        west: -180,
        east: -90,
        north: 90,
        south: -90
      }), new _Extent.default('EPSG:4326', {
        west: -90,
        east: 0,
        north: 90,
        south: -90
      }), new _Extent.default('EPSG:4326', {
        west: 0,
        east: 90,
        north: 90,
        south: -90
      }), new _Extent.default('EPSG:4326', {
        west: 90,
        east: 180,
        north: 90,
        south: -90
      })];
    } else {
      throw new Error("Unsupported panorama projection type ".concat(type, ".\n                Only ProjectionType.SPHERICAL and ProjectionType.CYLINDRICAL are supported"));
    }

    var builder = new _PanoramaTileBuilder.default(type, config.ratio || 1);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(PanoramaLayer).call(this, id, config.object3d || new THREE.Group(), schemeTile, builder, config));
    coordinates.xyz(_this.object3d.position);

    _this.object3d.quaternion.setFromUnitVectors(new THREE.Vector3(0, 0, 1), coordinates.geodesicNormal);

    _this.object3d.updateMatrixWorld(true); // FIXME: add CRS = '0' support


    _this.extent = new _Extent.default('EPSG:4326', {
      west: -180,
      east: 180,
      north: 90,
      south: -90
    });
    _this.disableSkirt = true;
    _this.options.segments = 8;
    _this.options.quality = 0.5;
    return _this;
  } // eslint-disable-next-line


  (0, _createClass2.default)(PanoramaLayer, [{
    key: "culling",
    value: function culling(node, camera) {
      return !camera.isBox3Visible(node.obb.box3D, node.obb.matrixWorld);
    }
    /**
     * Test the subdvision of a node, compared to this layer.
     *
     * @param {Object} context - The context of the update; see the {@link
     * MainLoop} for more informations.
     * @param {PlanarLayer} layer - This layer, parameter to be removed.
     * @param {TileMesh} node - The node to test.
     *
     * @return {boolean} - True if the node is subdivisable, otherwise false.
     */

  }, {
    key: "subdivision",
    value: function subdivision(context, layer, node) {
      if (node.level < 1) {
        return true;
      }

      var maxLevel = this.maxSubdivisionLevel || 10;

      if (maxLevel <= node.level) {
        return false;
      }

      var obb = node.obb;
      obb.updateMatrixWorld();
      var onScreen = context.camera.box3SizeOnScreen(obb.box3D, obb.matrixWorld);
      onScreen.min.z = 0;
      onScreen.max.z = 0;
      onScreen.getCenter(center); // give a small boost to central tiles

      var boost = 1 + Math.max(0, 1 - center.length());
      var dim = {
        x: 0.5 * (onScreen.max.x - onScreen.min.x) * context.camera.width,
        y: 0.5 * (onScreen.max.y - onScreen.min.y) * context.camera.height
      };
      var quality = layer.options.quality || 1.0;
      return boost * dim.x * quality >= textureSize.x && boost * dim.y * quality >= textureSize.y;
    }
  }]);
  return PanoramaLayer;
}(_TiledGeometryLayer2.default);

var _default = PanoramaLayer;
exports.default = _default;