"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPanoramaLayer = createPanoramaLayer;
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _View = _interopRequireDefault(require("../View"));

var _PanoramaLayer = _interopRequireDefault(require("./Panorama/PanoramaLayer"));

function createPanoramaLayer(id, coordinates, type) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  console.warn('createPanoramaLayer is deprecated, use the PanoramaLayer class instead.');
  return new _PanoramaLayer.default(id, coordinates, type, options);
}

function PanoramaView(viewerDiv, coordinates, type) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  THREE.Object3D.DefaultUp.set(0, 0, 1); // Setup View

  _View.default.call(this, coordinates.crs, viewerDiv, options); // Configure camera


  var camera = this.camera.camera3D;
  coordinates.xyz(camera.position);
  camera.fov = 45;
  camera.near = 0.1;
  camera.far = 1000;
  camera.up = coordinates.geodesicNormal; // look at to the north

  camera.lookAt(new THREE.Vector3(0, 1, 0).add(camera.position));

  if (camera.updateProjectionMatrix) {
    camera.updateProjectionMatrix();
  }

  camera.updateMatrixWorld();
  this.tileLayer = new _PanoramaLayer.default('panorama', coordinates, type, options);

  _View.default.prototype.addLayer.call(this, this.tileLayer);
}

PanoramaView.prototype = Object.create(_View.default.prototype);
PanoramaView.prototype.constructor = PanoramaView;

PanoramaView.prototype.addLayer = function (layer) {
  if (!layer) {
    return new Promise(function (resolve, reject) {
      return reject(new Error('layer is undefined'));
    });
  }

  if (layer.type != 'color') {
    throw new Error("Unsupported layer type ".concat(layer.type, " (PanoramaView only support 'color' layers)"));
  }

  return _View.default.prototype.addLayer.call(this, layer, this.tileLayer);
};

var _default = PanoramaView;
exports.default = _default;