"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var THREE = _interopRequireWildcard(require("three"));

var _LayeredMaterialConstants = require("../../../Renderer/LayeredMaterialConstants");

var _TiledGeometryLayer2 = _interopRequireDefault(require("../../../Layer/TiledGeometryLayer"));

var _PlanarTileBuilder = _interopRequireDefault(require("./PlanarTileBuilder"));

var PlanarLayer =
/*#__PURE__*/
function (_TiledGeometryLayer) {
  (0, _inherits2.default)(PlanarLayer, _TiledGeometryLayer);

  /**
   * A {@link TiledGeometryLayer} to use with a {@link PlanarView}. It has
   * specific method for updating and subdivising its grid.
   *
   * @constructor
   * @extends TiledGeometryLayer
   *
   * @param {string} id - The id of the layer, that should be unique. It is
   * not mandatory, but an error will be emitted if this layer is added a
   * {@link View} that already has a layer going by that id.
   * @param {Extent} extent - The extent to define the layer within.
   * @param {THREE.Object3d} [object3d=THREE.Group] - The object3d used to
   * contain the geometry of the TiledGeometryLayer. It is usually a
   * <code>THREE.Group</code>, but it can be anything inheriting from a
   * <code>THREE.Object3d</code>.
   * @param {Object} [config] - Optional configuration, all elements in it
   * will be merged as is in the layer. For example, if the configuration
   * contains three elements <code>name, protocol, extent</code>, these
   * elements will be available using <code>layer.name</code> or something
   * else depending on the property name.
   * @param {number} [config.maxSubdivisionLevel=5] - Maximum subdivision
   * level for this tiled layer.
   * @param {number} [config.maxDeltaElevationLevel=4] - Maximum delta between
   * two elevations tile.
   *
   * @throws {Error} <code>object3d</code> must be a valid
   * <code>THREE.Object3d</code>.
   */
  function PlanarLayer(id, extent, object3d) {
    var _this;

    var config = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    (0, _classCallCheck2.default)(this, PlanarLayer);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(PlanarLayer).call(this, id, object3d || new THREE.Group(), [extent], new _PlanarTileBuilder.default(), config));
    _this.extent = extent;
    return _this;
  } // eslint-disable-next-line


  (0, _createClass2.default)(PlanarLayer, [{
    key: "culling",
    value: function culling(node, camera) {
      return !camera.isBox3Visible(node.obb.box3D, node.obb.matrixWorld);
    }
    /**
     * Test the subdvision of a node, compared to this layer.
     *
     * @param {Object} context - The context of the update; see the {@link
     * MainLoop} for more informations.
     * @param {PlanarLayer} layer - This layer, parameter to be removed.
     * @param {TileMesh} node - The node to test.
     *
     * @return {boolean} - True if the node is subdivisable, otherwise false.
     */

  }, {
    key: "subdivision",
    value: function subdivision(context, layer, node) {
      var maxLevel = this.maxSubdivisionLevel || 5;
      var maxDeltaElevationLevel = this.maxDeltaElevationLevel || 4;

      if (maxLevel <= node.level) {
        return false;
      } // Prevent to subdivise the node if the current elevation level
      // we must avoid a tile, with level 20, inherits a level 3 elevation texture.
      // The induced geometric error is much too large and distorts the SSE


      var currentTexture = node.material.textures[_LayeredMaterialConstants.l_ELEVATION][0];

      if (currentTexture.extent) {
        var offsetScale = node.material.offsetScale[_LayeredMaterialConstants.l_ELEVATION][0];
        var ratio = offsetScale.z; // ratio is node size / texture size

        if (ratio < 1 / Math.pow(2, maxDeltaElevationLevel)) {
          return false;
        }
      }

      var onScreen = context.camera.box3SizeOnScreen(node.obb.box3D, node.matrixWorld); // onScreen.x/y/z are [-1, 1] so divide by 2
      // (so x = 0.4 means the object width on screen is 40% of the total screen width)

      var dim = {
        x: 0.5 * (onScreen.max.x - onScreen.min.x) * context.camera.width,
        y: 0.5 * (onScreen.max.y - onScreen.min.y) * context.camera.height
      }; // subdivide if on-screen width (and resp. height) is bigger than 30% of the screen width (resp. height)
      // TODO: the 30% value is arbitrary and needs to be configurable by the user
      // TODO: we might want to use texture resolution here as well

      return dim.x >= 256 && dim.y >= 256;
    }
  }]);
  return PlanarLayer;
}(_TiledGeometryLayer2.default);

var _default = PlanarLayer;
exports.default = _default;