"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _Layer2 = _interopRequireDefault(require("./Layer"));

var _LayeredMaterialNodeProcessing = require("../Process/LayeredMaterialNodeProcessing");

var _textureConverter = _interopRequireDefault(require("../Parser/textureConverter"));

var ElevationLayer =
/*#__PURE__*/
function (_Layer) {
  (0, _inherits2.default)(ElevationLayer, _Layer);

  /**
   * A simple layer, managing an elevation texture to add some reliefs on the
   * plane or globe view for example.
   *
   * @constructor
   * @extends Layer
   *
   * @param {string} id - The id of the layer, that should be unique. It is
   * not mandatory, but an error will be emitted if this layer is added a
   * {@link View} that already has a layer going by that id.
   * @param {Object} [config] - Optional configuration, all elements in it
   * will be merged as is in the layer. For example, if the configuration
   * contains three elements <code>name, protocol, extent</code>, these
   * elements will be available using <code>layer.name</code> or something
   * else depending on the property name.
   * @param {WMTSSource|WMSSource|WFSSource|TMSSource|FileSource} [config.source] data source
   *
   * @example
   * // Create an ElevationLayer
   * const elevation = new ElevationLayer('IGN_MNT', {
   *      source: {
   *          url: 'http://server.geo/wmts/SERVICE=WMTS&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL',
   *          protocol: 'wmts',
   *          format: 'image/x-bil;bits=32',
   *      },
   * });
   *
   * // Add the layer
   * view.addLayer(elevation);
   *
   * @example
   * // Add and create an ElevationLayer
   * view.addLayer({
   *     id: 'IGN_MNT',
   *     type: 'elevation',
   *     source: {
   *          url: 'http://server.geo/wmts/SERVICE=WMTS&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL',
   *          protocol: 'wmts',
   *          format: 'image/x-bil;bits=32',
   *     },
   * });
   */
  function ElevationLayer(id) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2.default)(this, ElevationLayer);
    return (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(ElevationLayer).call(this, id, 'elevation', config));
  }

  (0, _createClass2.default)(ElevationLayer, [{
    key: "update",
    value: function update(context, layer, node, parent) {
      return (0, _LayeredMaterialNodeProcessing.updateLayeredMaterialNodeElevation)(context, this, node, parent);
    }
  }, {
    key: "convert",
    value: function convert(data, extentDestination) {
      return _textureConverter.default.convert(data, extentDestination, this);
    }
  }]);
  return ElevationLayer;
}(_Layer2.default);

var _default = ElevationLayer;
exports.default = _default;