"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _TileGeometry = _interopRequireDefault(require("../Core/TileGeometry"));

var _TileMesh = _interopRequireDefault(require("../Core/TileMesh"));

var _LayeredMaterial = _interopRequireDefault(require("../Renderer/LayeredMaterial"));

var _Cache = _interopRequireDefault(require("../Core/Scheduler/Cache"));

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
var _default = {
  convert: function convert(requester, extent, layer) {
    var builder = layer.builder;
    var parent = requester;
    var level = parent !== undefined ? parent.level + 1 : 0;

    var _builder$computeShara = builder.computeSharableExtent(extent),
        sharableExtent = _builder$computeShara.sharableExtent,
        quaternion = _builder$computeShara.quaternion,
        position = _builder$computeShara.position;

    var south = sharableExtent.south().toFixed(6);
    var segment = layer.options.segments || 16;
    var key = "".concat(builder.type, "_").concat(layer.disableSkirt ? 0 : 1, "_").concat(segment, "_").concat(level, "_").concat(south);

    var geometry = _Cache.default.get(key); // build geometry if doesn't exist


    if (!geometry) {
      var paramsGeometry = {
        extent: sharableExtent,
        level: level,
        segment: segment,
        disableSkirt: layer.disableSkirt
      };
      geometry = new _TileGeometry.default(paramsGeometry, builder);

      _Cache.default.set(key, geometry);

      geometry._count = 0;

      geometry.dispose = function () {
        geometry._count--;

        if (geometry._count == 0) {
          THREE.BufferGeometry.prototype.dispose.call(geometry);

          _Cache.default.delete(key);
        }
      };
    } // build tile


    geometry._count++;
    var material = new _LayeredMaterial.default(layer.materialOptions);
    var tile = new _TileMesh.default(layer, geometry, material, extent, level); // TODO semble ne pas etre necessaire

    tile.layers.set(layer.threejsLayer);

    if (parent && parent instanceof _TileMesh.default) {
      // get parent extent transformation
      var pTrans = builder.computeSharableExtent(parent.extent); // place relative to his parent

      position.sub(pTrans.position).applyQuaternion(pTrans.quaternion.inverse());
      quaternion.premultiply(pTrans.quaternion);
    }

    tile.position.copy(position);
    tile.quaternion.copy(quaternion);
    tile.material.transparent = layer.opacity < 1.0;
    tile.material.uniforms.opacity.value = layer.opacity;
    tile.setVisibility(false);
    tile.updateMatrix();

    if (parent) {
      tile.setBBoxZ(parent.obb.z.min, parent.obb.z.max);
    } else if (layer.materialOptions && layer.materialOptions.useColorTextureElevation) {
      tile.setBBoxZ(layer.materialOptions.colorTextureElevationMinZ, layer.materialOptions.colorTextureElevationMaxZ);
    }

    tile.add(tile.obb);
    tile.material.setLightingOn(layer.lighting.enable);
    tile.material.uniforms.lightPosition.value = layer.lighting.position;

    if (layer.noTextureColor) {
      tile.material.uniforms.noTextureColor.value.copy(layer.noTextureColor);
    }

    return Promise.resolve(tile);
  }
};
exports.default = _default;