"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SIZE_TEXTURE_TILE = void 0;

var _Projection = _interopRequireDefault(require("../Core/Geographic/Projection"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

var SIZE_TEXTURE_TILE = 256;
exports.SIZE_TEXTURE_TILE = SIZE_TEXTURE_TILE;
var tileCoord = new _Extent.default('WMTS:WGS84G', 0, 0, 0);
var _default = {
  computeTileMatrixSetCoordinates: function computeTileMatrixSetCoordinates(tile, tileMatrixSet) {
    tileMatrixSet = tileMatrixSet || 'WGS84G';

    if (!(tileMatrixSet in tile.wmtsCoords)) {
      if (tile.wmtsCoords.WGS84G) {
        var c = tile.wmtsCoords.WGS84G[0];
        tileCoord.zoom = c.zoom;
        tileCoord.col = c.col;
        tileCoord.row = c.row;
      } else {
        _Projection.default.WGS84toWMTS(tile.extent, tileCoord);

        tile.wmtsCoords.WGS84G = [tileCoord.clone()];
      }

      tile.wmtsCoords[tileMatrixSet] = _Projection.default.getCoordWMTS_WGS84(tileCoord, tile.extent, tileMatrixSet);
    }
  },
  // The origin parameter is to be set to the correct value, bottom or top
  // (default being bottom) if the computation of the coordinates needs to be
  // inverted to match the same scheme as OSM, Google Maps or other system.
  // See link below for more information
  // https://alastaira.wordpress.com/2011/07/06/converting-tms-tile-coordinates-to-googlebingosm-tile-coordinates/
  computeTMSCoordinates: function computeTMSCoordinates(tile, extent) {
    var origin = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'bottom';
    extent = tile.extent.crs() == extent.crs() ? extent : extent.as(tile.extent.crs());
    var c = tile.extent.center();
    var layerDimension = extent.dimensions(); // Each level has 2^n * 2^n tiles...
    // ... so we count how many tiles of the same width as tile we can fit in the layer

    var tileCount = Math.round(layerDimension.x / tile.extent.dimensions().x); // ... 2^zoom = tilecount => zoom = log2(tilecount)

    var zoom = Math.floor(Math.log2(tileCount)); // Now that we have computed zoom, we can deduce x and y (or row / column)

    var x = (c.x() - extent.west()) / layerDimension.x;
    var y;

    if (origin == 'top') {
      y = (extent.north() - c.y()) / layerDimension.y;
    } else {
      y = (c.y() - extent.south()) / layerDimension.y;
    }

    return [new _Extent.default('TMS', zoom, Math.floor(y * tileCount), Math.floor(x * tileCount))];
  }
};
exports.default = _default;