"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

/* babel-plugin-inline-import './Chunk/pitUV.glsl' */
var pitUV = "vec2    pitUV(vec2 uvIn, vec4 pit)\n{\n    vec2  uv;\n    uv.x = uvIn.x* pit.z + pit.x;\n    uv.y = 1.0 -( (1.0 - uvIn.y) * pit.w + pit.y);\n\n    return uv;\n}\n\n";

/* babel-plugin-inline-import './Chunk/precision_qualifier.glsl' */
var precision_qualifier = "precision highp float;\nprecision highp int;\n";
var ShaderChunk = {
  pitUV: pitUV,
  precision_qualifier: precision_qualifier
};
/**
 * Install chunks in a target, for example THREE.ShaderChunk, with adding an
 * optional path.
 *
 * @param {Object} target - The target to install the chunks into.
 * @param {Object} chunks - The chunks to install. The key of each chunk will be
 * the name of installation of the chunk in the target (plus an optional path).
 * @param {string} [path] - A path to add before a chunk name as a prefix.
 *
 * @return {Object} The target with installed chunks.
 */

ShaderChunk.install = function (target, chunks, path) {
  if (!path) {
    return Object.assign(target, this);
  }

  Object.keys(chunks).forEach(function (key) {
    if (key == 'install') {
      return;
    }

    target[path + key] = chunks[key];
  });
  return target;
}; // Install all default shaders under the itowns


ShaderChunk.install(THREE.ShaderChunk, ShaderChunk, 'itowns/');
var _default = ShaderChunk;
exports.default = _default;