"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _togeojson = _interopRequireDefault(require("@mapbox/togeojson"));

var _Source2 = _interopRequireDefault(require("./Source"));

var _Fetcher = _interopRequireDefault(require("../Provider/Fetcher"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

var _GeoJsonParser = _interopRequireDefault(require("../Parser/GeoJsonParser"));

function getExtentFromGpxFile(file) {
  var bound = file.getElementsByTagName('bounds')[0];

  if (bound) {
    var west = bound.getAttribute('minlon');
    var east = bound.getAttribute('maxlon');
    var south = bound.getAttribute('minlat');
    var north = bound.getAttribute('maxlat');
    return new _Extent.default('EPSG:4326', west, east, south, north);
  }

  return new _Extent.default('EPSG:4326', -180, 180, -90, 90);
} // TODO move and refacto


function fileParser(text) {
  var parsedFile;
  var trimmedText = text.trim(); // We test the start of the string to choose a parser

  if (trimmedText.startsWith('<')) {
    // if it's an xml file, then it can be kml or gpx
    var parser = new DOMParser();
    var file = parser.parseFromString(text, 'application/xml');

    if (file.documentElement.tagName.toLowerCase() === 'kml') {
      parsedFile = _togeojson.default.kml(file);
    } else if (file.documentElement.tagName.toLowerCase() === 'gpx') {
      parsedFile = _togeojson.default.gpx(file);
      var line = parsedFile.features.find(function (e) {
        return e.geometry.type == 'LineString';
      });
      line.properties.stroke = 'red';
      parsedFile.extent = getExtentFromGpxFile(file);
    } else if (file.documentElement.tagName.toLowerCase() === 'parsererror') {
      throw new Error('Error parsing XML document');
    } else {
      throw new Error('Unsupported xml file, only valid KML and GPX are supported, but no <gpx> or <kml> tag found.', file);
    }
  } else if (trimmedText.startsWith('{') || trimmedText.startsWith('[')) {
    parsedFile = JSON.parse(text);

    if (parsedFile.type !== 'Feature' && parsedFile.type !== 'FeatureCollection') {
      throw new Error('This json is not a GeoJSON');
    }
  } else {
    throw new Error('Unsupported file: only well-formed KML, GPX or GeoJSON are supported');
  }

  return parsedFile;
}

var FileSource =
/*#__PURE__*/
function (_Source) {
  (0, _inherits2.default)(FileSource, _Source);

  /**
   * File source to use file in {@link GeometryLayer}, {@link ColorLayer} or {@link ElevationLayer}.
   * @constructor
   * @extends Source
   *
   * @param      {sourceParams}  source  The source
   * @param      {string}  source.projection  Data system projection, it needed to parse data
   * @param      {string}  crsOut  crd output data
   *
   * @example <caption>add geometry layer with geojson file </caption>
   * const ariege = new itowns.GeometryLayer('ariege', new itowns.THREE.Group());
   * ariege.update = itowns.FeatureProcessing.update;
   * ariege.convert = itowns.Feature2Mesh.convert({
   *      color: () => new itowns.THREE.Color(0xffcc00),
   *      extrude: () => 5000,
   * });
   * ariege.source = {
   *      protocol: 'file',
   *      url: 'https://raw.githubusercontent.com/gregoiredavid/france-geojson/master/departements/09-ariege/departement-09-ariege.geojson',
   *      projection: 'EPSG:4326',
   *      format: 'application/json',
   *      zoom: { min: 7, max: 7 },
   * };
   * view.addLayer(ariege);
   *
   * @example <caption>add color layer with geojson file </caption>
   * globeView.addLayer({
   *     type: 'color',
   *     id: 'Gpx',
   *     name: 'Ultra 2009',
   *     transparent: true,
   *     source: {
   *         url: 'https://raw.githubusercontent.com/iTowns/iTowns2-sample-data/master/ULTRA2009.gpx',
   *         protocol: 'file',
   *         projection: 'EPSG:4326',
   *     },
   * });
   *
   */
  function FileSource(source, crsOut) {
    var _this;

    (0, _classCallCheck2.default)(this, FileSource);

    if (!source.projection) {
      throw new Error('source.projection is required in FileSource');
    }

    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(FileSource).call(this, source));
    _this.url = source.url;
    _this.parsedData = [];
    _this.zoom = source.zoom || {
      min: 5,
      max: 21
    };
    var options = {
      buildExtent: true,
      crsIn: _this.projection,
      crsOut: crsOut,
      withNormal: !source.toTexture,
      withAltitude: !source.toTexture,
      mergeFeatures: true
    };
    _this.whenReady = _Fetcher.default.text(_this.url, source.networkOptions).then(fileParser).then(function (parsedFile) {
      return _GeoJsonParser.default.parse(parsedFile, options).then(function (feature) {
        feature.style = parsedFile.style;
        _this.parsedData = feature;
      });
    });
    return _this;
  }

  (0, _createClass2.default)(FileSource, [{
    key: "urlFromExtent",
    value: function urlFromExtent(extent) {
      return "".concat(this.url, ",").concat(extent.crs());
    }
  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit(extent) {
      var dataExtent = this.parsedData.extent;
      var localExtent = this.projection == extent.crs() ? extent : extent.as(this.projection);
      return (extent.zoom == undefined || !(extent.zoom < this.zoom.min || extent.zoom > this.zoom.max)) && dataExtent.intersectsExtent(localExtent);
    }
  }]);
  return FileSource;
}(_Source2.default);

var _default = FileSource;
exports.default = _default;