"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _construct2 = _interopRequireDefault(require("@babel/runtime/helpers/construct"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

/**
 * @typedef {Object} NetworkOptions - Options for fetching resources over the
 * network. For json or xml fetching, this object is passed as it is to fetch
 * as the init object, see [fetch documentation]{@link https://developer.mozilla.org/en-US/docs/Web/API/WindowOrWorkerGlobalScope/fetch#Parameters}.
 * @property {string} crossOrigin For textures, only this property is used. Its
 * value is directly assigned to the crossorigin property of html tags.
 * @property * Same properties as the init parameter of fetch
 */

/**
 * @typedef {object} sourceParams
 * @property {string} protocol source's protocol (wmts, wms, wfs, file, tms, static)
 * @property {string} url Base URL of the repository or of the file(s) to load
 * @property {NetworkOptions} [networkOptions = { crossOrigin: 'anonymous' }] the base url to fetch data source
 * @property {string} [projection] data's projection
 * @property {Extent} [extent] data's extent
 * @property {Attribution} [attribution] Attribution The intellectual property rights for the source
 * @property {string} [format] data format
 *
 */
var Source =
/*#__PURE__*/
function () {
  /**
   * Source are parameters to fetch source
   *
   * To extend {@link Source}, it is necessary to implement 2 functions:
   * {@link Source#urlFromExtent} and {@link Source#extentInsideLimit}
   * @param  {sourceParams}  source  object to set source
   *
   */
  function Source(source) {
    (0, _classCallCheck2.default)(this, Source);

    if (!source.protocol) {
      throw new Error('New Source: protocol is required');
    }

    if (!source.url) {
      throw new Error('New Source: url is required');
    }

    this.format = source.format;
    this.protocol = source.protocol;
    this.networkOptions = source.networkOptions || {
      crossOrigin: 'anonymous'
    };
    this.projection = source.projection;
    this.attribution = source.attribution;

    if (source.extent && !(source.extent instanceof _Extent.default)) {
      if (Array.isArray(source.extent)) {
        this.extent = (0, _construct2.default)(_Extent.default, [this.projection].concat((0, _toConsumableArray2.default)(source.extent)));
      } else {
        this.extent = new _Extent.default(this.projection, source.extent);
      }
    } else {
      this.extent = source.extent;
    }
  }

  (0, _createClass2.default)(Source, [{
    key: "handlingError",
    value: function handlingError(err) {
      console.warn("err ".concat(this), err);
    }
    /**
     * Generate url from extent. This url is link to fetch data inside the extent.
     *
     * @param      {Extent}  extent  extent to convert in url
     * @return     {string}  url from extent
     */
    // eslint-disable-next-line

  }, {
    key: "urlFromExtent",
    value: function urlFromExtent() {
      throw new Error('In extented Source, you have to implement the method urlFromExtent!');
    }
    /**
     * test if the extent is inside data's source
     *
     * @param      {Extent}   extent extent to test
     * @return     {boolean}  return of test
     */
    // eslint-disable-next-line

  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit() {
      throw new Error('In extented Source, you have to implement the method extentInsideLimit!');
    }
    /**
     * test if all extents are inside data's source
     *
     * @param      {Array.<Extent>}   extents  The extents to test
     * @return     {boolean}  test result
     */

  }, {
    key: "extentsInsideLimit",
    value: function extentsInsideLimit(extents) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = extents[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var extent = _step.value;

          if (!this.extentInsideLimit(extent)) {
            return false;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return true;
    }
  }]);
  return Source;
}();

var _default = Source;
exports.default = _default;