"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _construct2 = _interopRequireDefault(require("@babel/runtime/helpers/construct"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _flatbush = _interopRequireDefault(require("flatbush"));

var _Source2 = _interopRequireDefault(require("./Source"));

var _Fetcher = _interopRequireDefault(require("../Provider/Fetcher"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

function _selectImagesFromSpatialIndex(index, images, extent) {
  return index.search(extent.west(), extent.south(), extent.east(), extent.north()).map(function (i) {
    return images[i];
  });
}

function buildUrl(layer, image) {
  return layer.url.href.substr(0, layer.url.href.lastIndexOf('/') + 1) + image;
}

var StaticSource =
/*#__PURE__*/
function (_Source) {
  (0, _inherits2.default)(StaticSource, _Source);

  /**
   * Images source to panorama layer {@link PanoramaLayer}
   *
   * @constructor
   * @extends Source
   * @param {sourceParams}  source  The source
   * @param {string}  source.extent It's extent of panoramic's images
  */
  function StaticSource(source) {
    var _this;

    (0, _classCallCheck2.default)(this, StaticSource);

    if (!source.extent) {
      throw new Error('layer.extent is required');
    }

    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(StaticSource).call(this, source));
    _this.zoom = {
      min: 0,
      max: 0
    };
    _this.url = new URL(source.url, window.location);
    _this.whenReady = _Fetcher.default.json(_this.url.href).then(function (metadata) {
      _this.images = []; // eslint-disable-next-line guard-for-in

      for (var image in metadata) {
        var extent = (0, _construct2.default)(_Extent.default, [_this.projection].concat((0, _toConsumableArray2.default)(metadata[image])));

        _this.images.push({
          image: image,
          extent: extent
        });
      }

      if (!_this.images.length) {
        return;
      }

      _this._spatialIndex = new _flatbush.default(_this.images.length);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = _this.images[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _image = _step.value;

          _this._spatialIndex.add(_image.extent.west(), _image.extent.south(), _image.extent.east(), _image.extent.north());
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      _this._spatialIndex.finish();
    }).then(function () {
      if (!_this.format) {
        // fetch the first image to detect format
        if (_this.images.length) {
          var url = buildUrl((0, _assertThisInitialized2.default)((0, _assertThisInitialized2.default)(_this)), _this.images[0].image);
          return fetch(url, _this.networkOptions).then(function (response) {
            _this.format = response.headers.get('Content-type');

            if (_this.format === 'application/octet-stream') {
              _this.format = 'image/x-bil';
            }

            if (!_this.format) {
              throw new Error("".concat(_this.name, ": could not detect layer format, please configure 'layer.format'."));
            }
          });
        }
      }
    });
    return _this;
  }

  (0, _createClass2.default)(StaticSource, [{
    key: "urlFromExtent",
    value: function urlFromExtent(extent) {
      var selection = this.getSourceExtents(extent);
      return this.url.href.substr(0, this.url.href.lastIndexOf('/') + 1) + selection.image;
    }
  }, {
    key: "handlingError",
    value: function handlingError(err, url) {
      console.error("Source static: ".concat(this.url, ", ").concat(err.response.status, " error while trying to fetch data. Url was ").concat(url, "."), err);
    }
  }, {
    key: "canTileTextureBeImproved",
    value: function canTileTextureBeImproved(extent, texture) {
      if (!this.images) {
        return false;
      }

      var s = this.getSourceExtents(extent);

      if (!s) {
        return false;
      }

      if (!texture.image) {
        return true;
      }

      var urlTexture = texture.image.currentSrc;
      var urlSource = this.url.href.substr(0, this.url.href.lastIndexOf('/') + 1) + s.image;
      return urlSource != urlTexture;
    }
  }, {
    key: "getSourceExtents",
    value: function getSourceExtents(extent) {
      // select the smallest image entirely covering the tile
      extent = extent.crs() === this.extent.crs() ? extent : extent.as(this.extent.crs());

      var candidates = _selectImagesFromSpatialIndex(this._spatialIndex, this.images, extent);

      var selection;
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = candidates[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var entry = _step2.value;

          if (extent.isInside(entry.extent)) {
            if (!selection) {
              selection = entry;
            } else {
              var d = selection.extent.dimensions();
              var e = entry.extent.dimensions();

              if (e.x <= d.x && e.y <= d.y) {
                selection = entry;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return selection;
    }
  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit(extent) {
      return extent.intersectsExtent(this.extent);
    }
  }]);
  return StaticSource;
}(_Source2.default);

var _default = StaticSource;
exports.default = _default;