"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _Source2 = _interopRequireDefault(require("./Source"));

var _URLBuilder = _interopRequireDefault(require("../Provider/URLBuilder"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

var TMSSource =
/*#__PURE__*/
function (_Source) {
  (0, _inherits2.default)(TMSSource, _Source);

  /**
   * Tiled images source
   * @constructor
   * @extends Source
   * @param {sourceParams}  source
   * @param {string} [source.origin] origin row coordinate: 'top' or 'bottom'
   * @param {Object} [source.zoom]
   * @param {number} [source.zoom.min] layer's zoom minimum
   * @param {number} [source.zoom.max] layer's zoom maximum
   * @param {string} [source.tileMatrixSet='WGS84']  define tile matrix set of tms layer (ex: 'PM', 'WGS84')
   *
   * @example <caption>Add color layer with tms source</caption>
   * const colorlayer = new ColorLayer('OPENSM', {
   *     source: {
   *         protocol: 'xyz',
   *         format: 'image/png',
   *         url: 'http://osm.io/styles/${z}/${x}/${y}.png',
   *         attribution: {
   *             name: 'OpenStreetMap',
   *             url: 'http://www.openstreetmap.org/',
   *         },
   *         tileMatrixSet: 'PM',
   *     }
   * });
   * // Add the layer
   * view.addLayer(colorlayer);
  */
  function TMSSource(source) {
    var _this;

    (0, _classCallCheck2.default)(this, TMSSource);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(TMSSource).call(this, source));

    if (!source.extent) {
      // default to the full 3857 extent
      _this.extent = new _Extent.default('EPSG:3857', -20037508.342789244, 20037508.342789244, -20037508.342789255, 20037508.342789244);
    }

    _this.zoom = source.zoom || {
      min: 0,
      max: 18
    };
    _this.origin = source.origin || (source.protocol == 'xyz' ? 'top' : 'bottom');
    _this.format = _this.format || 'image/png';
    _this.url = source.url;

    if (source.tileMatrixSet) {
      _this.tileMatrixSet = source.tileMatrixSet;
    } else if (_this.projection == 'EPSG:3857') {
      _this.tileMatrixSet = 'PM';
    } else {
      _this.tileMatrixSet = 'WGS84';
    }

    return _this;
  }

  (0, _createClass2.default)(TMSSource, [{
    key: "urlFromExtent",
    value: function urlFromExtent(extent) {
      return _URLBuilder.default.xyz(extent, this);
    }
  }, {
    key: "handlingError",
    value: function handlingError(err) {
      console.warn("err ".concat(this.url), err);
    }
  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit(extent) {
      // This layer provides data starting at level = layer.source.zoom.min
      // (the zoom.max property is used when building the url to make
      //  sure we don't use invalid levels)
      // TODO: add extent limit
      return extent.zoom >= this.zoom.min && extent.zoom <= this.zoom.max;
    }
  }]);
  return TMSSource;
}(_Source2.default);

var _default = TMSSource;
exports.default = _default;