"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _Source2 = _interopRequireDefault(require("./Source"));

var _URLBuilder = _interopRequireDefault(require("../Provider/URLBuilder"));

var WMSSource =
/*#__PURE__*/
function (_Source) {
  (0, _inherits2.default)(WMSSource, _Source);

  /**
   * Images source
   * @constructor
   * @extends Source
   * @param {sourceParams}  source
   * @param {string} source.name name of layer wms
   * @param {Extent} source.extent extent of wms source
   * @param {string} [source.style=''] style of layer wms
   * @param {number} [source.heightMapWidth=256] size texture in pixel
   * @param {string} [source.version='1.3.0'] wms version
   * @param {string} [source.axisOrder] wms axis order ('wsen' or 'swne')
   * @param {boolean} [source.transparent=false] source return texture with transparence
   * @param {Object} [source.zoom]
   * @param {number} [source.zoom.min] layer's zoom minimum
   * @param {number} [source.zoom.max] layer's zoom maximum
   *
   * @example <caption>Add color layer with wms source</caption>
   * const colorlayer = new ColorLayer('Region', {
   *     source: {
   *         url: 'https://wxs.fr/wms',
   *         protocol: 'wms',
   *         version: '1.3.0',
   *         name: 'REGION.2016',
   *         style: '',
   *         projection: 'EPSG:3857',
   *         extent: {
   *             west: '-6880639.13557728',
   *             east: '6215707.87974825',
   *             south: '-2438399.00148845',
   *             north: '7637050.03850605',
   *         },
   *         transparent: true,
   *     },
   * });
   * // Add the layer
   * view.addLayer(colorlayer);
   *
  */
  function WMSSource(source) {
    var _this;

    (0, _classCallCheck2.default)(this, WMSSource);

    if (!source.name) {
      throw new Error('source.name is required.');
    }

    if (!source.extent) {
      throw new Error('source.extent is required');
    }

    if (!source.projection) {
      throw new Error('source.projection is required');
    }

    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(WMSSource).call(this, source));
    _this.name = source.name;
    _this.zoom = source.zoom || {
      min: 0,
      max: 21
    };
    _this.format = _this.format || 'image/png';
    _this.style = source.style || '';
    _this.width = source.heightMapWidth || 256;
    _this.version = source.version || '1.3.0';
    _this.transparent = source.transparent || false;

    if (!source.axisOrder) {
      // 4326 (lat/long) axis order depends on the WMS version used
      if (source.projection == 'EPSG:4326') {
        // EPSG 4326 x = lat, long = y
        // version 1.1.0 long/lat while version 1.3.0 mandates xy (so lat,long)
        _this.axisOrder = _this.version === '1.1.0' ? 'wsen' : 'swne';
      } else {
        // xy,xy order
        _this.axisOrder = 'wsen';
      }
    }

    var crsPropName = _this.version === '1.3.0' ? 'CRS' : 'SRS';
    _this.url = "".concat(source.url, "?SERVICE=WMS&REQUEST=GetMap&LAYERS=").concat(_this.name, "&VERSION=").concat(_this.version, "&STYLES=").concat(_this.style, "&FORMAT=").concat(_this.format, "&TRANSPARENT=").concat(_this.transparent, "&BBOX=%bbox&").concat(crsPropName, "=").concat(_this.projection, "&WIDTH=").concat(_this.width, "&HEIGHT=").concat(_this.width);
    return _this;
  }

  (0, _createClass2.default)(WMSSource, [{
    key: "urlFromExtent",
    value: function urlFromExtent(extent) {
      return _URLBuilder.default.bbox(extent, this);
    }
  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit(extent) {
      var localExtent = this.projection == extent.crs() ? extent : extent.as(this.projection);
      return (extent.zoom == undefined || !(extent.zoom < this.zoom.min || extent.zoom > this.zoom.max)) && this.extent.intersectsExtent(localExtent);
    }
  }]);
  return WMSSource;
}(_Source2.default);

var _default = WMSSource;
exports.default = _default;