"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _Source2 = _interopRequireDefault(require("./Source"));

var _URLBuilder = _interopRequireDefault(require("../Provider/URLBuilder"));

var WMTSSource =
/*#__PURE__*/
function (_Source) {
  (0, _inherits2.default)(WMTSSource, _Source);

  /**
   * Tiles images source
   * @constructor
   * @extends Source
   * @param {sourceParams}  source
   * @param {string} source.name name of layer wmts
   * @param {string} source.tileMatrixSet  define tile matrix set of wmts layer (ex: 'PM', 'WGS84')
   * @param {Array.<Object>} [source.tileMatrixSetLimits] The limits for the tile matrix set
   * @param {number} source.tileMatrixSetLimits.minTileRow Minimum row for tiles at the level
   * @param {number} source.tileMatrixSetLimits.maxTileRow Maximum row for tiles at the level
   * @param {number} source.tileMatrixSetLimits.minTileCol Minimum col for tiles at the level
   * @param {number} source.tileMatrixSetLimits.maxTileCol Maximum col for tiles at the level
   * @param {Object} [source.zoom]
   * @param {number} [source.zoom.min] layer's zoom minimum
   * @param {number} [source.zoom.max] layer's zoom maximum
   *
   * @example <caption>Add color layer with wmts source</caption>
   * const colorlayer = new ColorLayer('darkmap', {
   *     source: {
   *          protocol: 'wmts',
   *          name: 'DARK',
   *          tileMatrixSet: 'PM',
   *          url: 'http://server.geo/wmts',
   *          format: 'image/jpg',
   *     }
   * });
   * // Add the layer
   * view.addLayer(colorlayer);
   *
   */
  function WMTSSource(source) {
    var _this;

    (0, _classCallCheck2.default)(this, WMTSSource);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(WMTSSource).call(this, source));

    if (!source.name) {
      throw new Error('New WMTSSource: name is required');
    }

    _this.format = _this.format || 'image/png';
    _this.version = source.version || '1.0.0';
    _this.tileMatrixSet = source.tileMatrixSet || 'WGS84';
    _this.style = source.style || 'normal';
    _this.name = source.name;
    _this.url = "".concat(source.url) + "?LAYER=".concat(_this.name) + "&FORMAT=".concat(_this.format) + '&SERVICE=WMTS' + "&VERSION=".concat(_this.version) + '&REQUEST=GetTile' + "&STYLE=".concat(_this.style) + "&TILEMATRIXSET=".concat(_this.tileMatrixSet) + '&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL';
    _this.zoom = source.zoom;
    _this.tileMatrixSetLimits = source.tileMatrixSetLimits; // If the projection is undefined,
    // It is deduced from the tileMatrixSet,
    // The projection is coherent with the projection

    if (!_this.projection) {
      if (_this.tileMatrixSet === 'WGS84' || _this.tileMatrixSet === 'WGS84G') {
        _this.projection = 'EPSG:4326';
      } else {
        _this.projection = 'EPSG:3857';
      }
    }

    if (!_this.zoom) {
      if (_this.tileMatrixSetLimits) {
        var arrayLimits = Object.keys(_this.tileMatrixSetLimits);
        var size = arrayLimits.length;
        var maxZoom = Number(arrayLimits[size - 1]);
        _this.zoom = {
          min: maxZoom - size + 1,
          max: maxZoom
        };
      } else {
        _this.zoom = {
          min: 2,
          max: 20
        };
      }
    }

    return _this;
  }

  (0, _createClass2.default)(WMTSSource, [{
    key: "urlFromExtent",
    value: function urlFromExtent(extent) {
      return _URLBuilder.default.xyz(extent, this);
    }
  }, {
    key: "extentInsideLimit",
    value: function extentInsideLimit(extent) {
      return extent.zoom >= this.zoom.min && extent.zoom <= this.zoom.max && (this.tileMatrixSetLimits == undefined || extent.row >= this.tileMatrixSetLimits[extent.zoom].minTileRow && extent.row <= this.tileMatrixSetLimits[extent.zoom].maxTileRow && extent.col >= this.tileMatrixSetLimits[extent.zoom].minTileCol && extent.col <= this.tileMatrixSetLimits[extent.zoom].maxTileCol);
    }
  }]);
  return WMTSSource;
}(_Source2.default);

var _default = WMTSSource;
exports.default = _default;